/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctDrawBoard.cpp
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

/*------------------------------------------------------------- Includes */

#include "ctDrawBoard.h"
#include "ctPieces.h"
#include "ctPixmaps.h"
#include "ctSignalRouter.h"

#include <qpainter.h>
#include <qpen.h>
#include <qevent.h>
#include <qfiledialog.h>
#include <qregexp.h>
#include <qclipboard.h>
#include <qapplication.h>

/*------------------------------------------------------------ Functions */

/** Constructor for the class \a ctDrawBoard.
@param parent Pointer to the parent widget
@param name Internal name for the new \a ctDrawBoard
*/
ctDrawBoard::ctDrawBoard( QWidget *parent, const char *name )
    : QWidget( parent, name ),
      sTaskNumber(""),
      iPiece(PAWN),
      bEnabled(false)
{
  setBackgroundColor(Qt::white);
  setFixedSize(300, 300);
  setupConnects();
}

/** Sets up the connects for this widget. */
void ctDrawBoard::setupConnects()
{

/**************************/  
/* Signal router -> Board */
/**************************/  

  connect(srChessTask, SIGNAL( sigClearBoard() ),
          this, SLOT( slotClearBoard() ));
  connect(srChessTask, SIGNAL( sigInitialPosition() ),
          this, SLOT( slotInitialPosition() ));

}


/** Repaints the widget, i.e. the board with the actual chess position.
*/
void ctDrawBoard::paintEvent(QPaintEvent *)
{
  // Painter device
  QPainter *paint = new QPainter();
  
  // Is the board enabled?
  if (bEnabled == true)
  {
    // Yes
    drawCurrentBoard(paint);
  }
  else
  {
    // No
    // Start painting to widget
    paint->begin( this );

    // Draw ChessTask logo (assuming a size of 300x300 pixels)
    paint->drawPixmap(61, 127, pixMaps[44]);
   
    // End painting		    
    paint->end();
  }

  delete paint;
}


/** Draws the column letter with the given \a index onto the widget area.
@param paint Painter for the widget
@param index Index of the column (0--7)
*/
void ctDrawBoard::drawColumnLetter(QPainter *paint, 
                                   unsigned int index)
{
  QRect labelRect;
  QString sLetter("A");

  sLetter[0] = 65+index;

  labelRect = paint->boundingRect(X_MARGIN, Y_MARGIN, 29, 29, 
                                  QPainter::AlignTop | QPainter::AlignLeft, 
				  sLetter);
  paint->drawText(X_MARGIN+index*29+(29-labelRect.width())/2, COLUMNS_Y,
                  labelRect.width()+1, labelRect.height()+1,
		  QPainter::AlignTop | QPainter::AlignLeft, 
		  sLetter);
}

/** Draws the row letter with the given \a index onto the widget area.
@param paint Painter for the widget
@param index Index of the row (0--7)
*/
void ctDrawBoard::drawRowLetter(QPainter *paint, unsigned int index)
{
  QRect labelRect;
  QString sLetter("1");

  sLetter[0] = 49+(7-index);

  labelRect = paint->boundingRect(X_MARGIN, Y_MARGIN, 29, 29, 
                                  QPainter::AlignTop | QPainter::AlignLeft, 
				  sLetter);
  paint->drawText(ROWS_X, Y_MARGIN+index*29+(29-labelRect.height())/2,
                  labelRect.width()+1, labelRect.height()+1,
		  QPainter::AlignTop | QPainter::AlignLeft, 
		  sLetter);
}

/** Draws the appropriate bitmap for the given piece \a index at \a row and
\a col of the board.
@param paint Painter for the widget
@param index Index of the bitmap
@param row The row
@param col The column
*/
void ctDrawBoard::drawPieceBitmap(QPainter *paint, 
                                  unsigned int index, 
                                  unsigned int row, 
				  unsigned int col)
{
  if (((row*9 + col) % 2) == 0)
  {
    // White square
    paint->drawPixmap(X_MARGIN+29*col, Y_MARGIN+29*row, pixMaps[index]);
  }
  else
  {
    // Black square (we have 17 different pixmaps)
    paint->drawPixmap(X_MARGIN+29*col, Y_MARGIN+29*row, pixMaps[index + 17]);
  }
}

/** Draws the number of the current task and the total number on top
of the board.
@param paint Painter for the widget
*/
void ctDrawBoard::drawTaskNumber(QPainter *paint)
{
  QRect labelRect;

  if (sTaskNumber.isEmpty())
  {  
    return;
  }

  labelRect = paint->boundingRect(150, 0, 150, 25, 
                                  QPainter::AlignTop | QPainter::AlignLeft, 
				  sTaskNumber);
  paint->drawText(150-labelRect.width()/2, 0,
                  labelRect.width()+1, labelRect.height()+1,
		  QPainter::AlignTop | QPainter::AlignLeft, 
		  sTaskNumber);
}

/** Draws the current chess board onto the widget area.
@param paint Painter for the widget
*/
void ctDrawBoard::drawCurrentBoard(QPainter *paint)
{
  unsigned int row, col;
  
  // Pen with color black and solid line
  QPen gridPen(black, 2);
  
  // Set new pen
  paint->begin( this );
  paint->setPen( gridPen );
  
  for (row = 0; row < 8; row++)
  {
    for (col = 0; col < 8; col++)
    {
      drawPieceBitmap(paint, iPosition(row, col), row, col);
    }
  }  
  // Draw frame
  paint->drawRect(X_MARGIN-1, Y_MARGIN-1, 235, 235);

  // Draw Notation for rows and columns
  for (col = 0; col < 8; col++)
  {
    drawColumnLetter(paint, col);
  }  
  for (row = 0; row < 8; row++)
  {
    drawRowLetter(paint, row);
  }  
  
  drawTaskNumber(paint);

  paint->end();
}

/** Here we overload the default handler for MousePress
events, in order to detect when a new piece is set, i.e.
has to be drawn.
@param event Pointer to the \a QMouseEvent
*/
void ctDrawBoard::mousePressEvent(QMouseEvent *event)
{

  // Is the board not enabled? -> do nothing
  if (bEnabled == false)
    return;

  // Is the mouse pointer on the board area?
  if ((event->x() > X_MARGIN) && (event->x() < (X_MARGIN+232)) && 
      (event->y() > Y_MARGIN) && (event->y() < (Y_MARGIN+232)))
  {
    // Yes, so calculate array index
    unsigned int row = ((event->y() - Y_MARGIN) / 29);
    unsigned int col = ((event->x() - X_MARGIN) / 29);

    // Painter device
    QPainter *paint = new QPainter();
    // Begin painting
    paint->begin( this );

    // Do we have to draw or erase?
    if (event->button() == QMouseEvent::LeftButton)
    {
      // Draw
      if (iPosition(row, col) != iPiece)
      {
        iPosition(row, col) = iPiece;
	drawPieceBitmap(paint, iPiece, row, col);
      }	
    }
    else
    {
      // Erase
      if (iPosition(row, col) != EMPTY)
      {
        iPosition(row, col) = EMPTY;
	drawPieceBitmap(paint, EMPTY, row, col);
      }	
    }  
    
    // End painting
    paint->end();

    // Delete painter
    delete paint;
  }
}

/*---------------------------------------------------------------- Slots */

/** Sets the current \a ctPosition.
@param positionNew The new position
*/
void ctDrawBoard::slotSetPosition(const ctPosition &positionNew)
{
  if (bEnabled == false)
  {
    return;
  }
  
  iPosition = positionNew;

  update();
}

/** Sets the new task number string.
@param stringNew The new string
*/
void ctDrawBoard::slotSetTaskNumber(const QString &stringNew)
{
  if (bEnabled == false)
  {
    return;
  }
  
  sTaskNumber = stringNew;

  update();
}

/** Sets the status of the board to either
enabled (=``true'') or disabled (=``false''). If the
board is disabled the \a ChessTask logo is displayed
instead.
@param statusNew The new status
*/
void ctDrawBoard::slotSetBoardEnable(bool statusNew)
{
  bEnabled = statusNew;

  update();
}

/** Emits the signal \a sigSetPosition
for setting the current position.
*/
void ctDrawBoard::slotGetPosition()
{
  emit sigSetPosition(iPosition);
}

/** Sets the current piece to the given
pixmap ID.
@param pixmapID ID of the piece in the pixMaps ``vector''
*/
void ctDrawBoard::slotSetPiece(unsigned int pixmapID)
{
  iPiece = pixmapID;
}

/** Clears the current board.
*/
void ctDrawBoard::slotClearBoard()
{
  if (bEnabled == false)
  {
    return;
  }
  
  iPosition.clearBoard();

  update();
}

/** Sets the current board to
the initial position.
*/
void ctDrawBoard::slotInitialPosition()
{
  if (bEnabled == false)
  {
    return;
  }
  
  iPosition.initBoard();

  update();
}

/** Copies the current position to the application's
clipboard in LaTeX format. From there it can be pasted
into other applications, e.g. a text editor.
*/
void ctDrawBoard::slotCopyLaTeXPosition()
{
  // Do nothing if the board is not enabled
  if (bEnabled == false)
  {
    return;
  }
  
  // String that takes the position
  QString clipPosition;
  // TextStream for writing to the string
  QTextStream t(clipPosition, IO_WriteOnly);
  
  // Writing the position
  iPosition.exportLaTeX(t);

  // Copy the text to the clipboard
  QApplication::clipboard()->setText(clipPosition);
  
}

/** Copies the current position to the application's
clipboard in HTML format. From there it can be pasted
into other applications, e.g. a text editor.
@param graphicsDir Reference to the string containing the ``graphics''
directory
*/
void ctDrawBoard::slotCopyHTMLPosition(const QString &graphicsDir)
{
  // Do nothing if the board is not enabled
  if (bEnabled == false)
  {
    return;
  }
  
  // String that takes the position
  QString clipPosition;
  // TextStream for writing to the string
  QTextStream t(clipPosition, IO_WriteOnly);
  
  // Writing the position
  iPosition.exportHTML(t, graphicsDir);

  // Copy the text to the clipboard
  QApplication::clipboard()->setText(clipPosition);
  
}

